
CombatAlert = Rock:NewAddon("CombatAlert", "LibRockDB-1.0", "LibRockEvent-1.0", "LibRockTimer-1.0", "LibRockConfig-1.0", "LibRockConsole-1.0", "LibParser-4.0", "LibFuBarPlugin-3.0")

local SM = Rock("LibSharedMedia-2.0")

self = CombatAlert

self.locked = true

self:SetDatabase('CombatAlertDB', 'CombatAlertPCDB')

self:SetFuBarOption('iconPath', "Interface\\AddOns\\CombatAlert\\icon")

self:AddSlashCommand('OnCommand', '/CombatAlert', '/cal', '/ca')

local MESSAGE_NORMAL = 1
local MESSAGE_WARNING = 2
local MESSAGE_CRITICAL = 3
local COOLDOWN_MINIMUM = 3
local COOLDOWN_MAXIMUM = nil
local COOLDOWN_MODIFIERS = { -- TODO: add modifier check
	["Cold Snap"] = {
		[3] = "Frost Nova",
		[1] = "Ice Block",
		[2] = "Frost Ward",
		[4] = "Ice Barrier",
		[5] = "Cone of Cold",
	},
	["Preparation"] = {
		[1] = "Evasion",
		[2] = "Sprint",
		[3] = "Vanish",
		[4] = "Cold Blood",
		[5] = "Adrenaline Rush",
		[6] = "Premeditation",
	},
	["Renataki's Charm of Beasts"] = {
		[1] = "Arcane Shot",
		[2] = "Aimed Shot",
		[3] = "Multi-Shot",
		[4] = "Volley",
	}
}
local COOLDOWN_GROUPS = { -- TODO: add group check
	["Arcane/Aimed Shots"] = {
		[1] = "Arcane Shot",
		[2] = "Aimed Shot",
	},
	["Reta/SW/Reck"] = {
		[1] = "Retaliation",
		[2] = "Shield Wall",
		[3] = "Recklessness",
	},
	["Shocks"] = {
		[1] = "Earth Shock",
		[2] = "Flame Shock",
		[3] = "Frost Shock",
	},
	["Traps"] = {
		[1] = "Freezing Trap",
		[2] = "Immolation Trap",
		[3] = "Explosive Trap",
		[4] = "Frost Trap",
	},
	["Wards"] = {
		[1] = "Frost Ward",
		[2] = "Fire Ward",
	}
}
local COOLDOWN_SCHOOLS = { -- TODO: add school check
}

function CombatAlert:OnInitialize()
	self:SetDatabaseDefaults('profile', {
		preset = false,
		unknown = false,
		compact = false,
		collect = false,
		silent = false,
		x = 0,
		y = 150,
		frame = "default",
		fontsize = "32",
		fonttype = "Skurri",
		fontoutline = "OUTLINE",
		messagecolor = { -- TODO: recolor defaults
			state = {r=1.0, g=1.0, b=1.0, a=1.0},
			cooldown = {r=0.5, g=0.5, b=0.5, a=1.0},
			debuff = {r=0.75, g=0.75, b=0.75, a=1.0},
			buff = {r=1.0, g=1.0, b=1.0, a=1.0},
			cast = {r=1.0, g=1.0, b=0.0, a=1.0},
			fade = {r=0.5, g=0.5, b=0.5, a=1.0},
			dispel = {r=0.5, g=0.5, b=0.5, a=1.0},
		},
		messagesounds = { -- enable sharedmedia
			cooldown = "default",
			warn = "default",
			crit = "default",
		},
	})
	self:SetDatabaseDefaults('char', {
		cooldowns = {},
		known = {},
		show = {},
		warn = {},
		crit = {},
	})
	self:SetConfigTable({
		type = 'group',
		name = "Combat Alert",
		desc = "Combat alert announcer",
		icon = "Interface\\AddOns\\CombatAlert\\icon",
		args = {
			preset = {
				order = 100,
				type = 'boolean',
				name = 'Preset mode',
				desc = 'Enable preset event alerts',
				get = function(preset)
					return self.db.profile.preset
				end,
				set = function(preset)
					self.db.profile.preset = preset
					self.db.profile.unknown = preset
				end,
			},
			unknown = {
				order = 101,
				type = 'boolean',
				name = 'Allow unknown',
				desc = 'Enable preset unknown unit events',
				get = function()
					return self.db.profile.unknown
				end,
				set = function(unknown)
					self.db.profile.unknown = unknown
				end,
				hidden = function()
					return self.db.profile.preset
				end,
			},
			compact = {
				order = 102,
				type = 'boolean',
				name = 'Compact messages',
				desc = 'Enable displaying compact alerts',
				get = function()
					return self.db.profile.compact
				end,
				set = function(compact)
					self.db.profile.compact = compact
				end,
			},
			collect = {
				order = 104,
				type = 'boolean',
				name = 'Collect alerts',
				desc = 'Enable new alert collection',
				get = function()
					return self.db.profile.collect
				end,
				set = function(collect)
					self.db.profile.collect = collect
				end,
			},
			silent = {
				order = 105,
				type = 'boolean',
				name = 'Silent collect',
				desc = 'Disable printing new alerts',
				get = function()
					return self.db.profile.silent
				end,
				set = function(silent)
					self.db.profile.silent = silent
				end,
				hidden = function()
					return not self.db.profile.collect
				end,
			},
			frame = {
				order = 109,
				type = 'choice',
				name = 'Message frame',
				desc = 'Choose the message frame to use',
				get = function()
					return self.db.profile.frame
				end,
				set = function(frame)
					if (frame == "sct" and IsAddOnLoaded("sct")) or (frame == "msbt" and IsAddOnLoaded("MikScrollingBattleText")) or (frame == "blizzard" and UIOptionsFrameCheckButtons["SHOW_COMBAT_TEXT_TEXT"].value == "1") or frame == "default" then
						self.db.profile.frame = frame
					elseif frame == "parrot" and IsAddOnLoaded("Parrot") then
						self:Print("Parrot can replace this addon")
						self.db.profile.frame = frame
					else
						self:Print("frame isnt usable!")
					end
					if self.db.profile.frame ~= "default" and not self.locked then
						self:ToggleMessageFrame()
					end
				end,
				choices = {
					["blizzard"] = "Blizzard",
					["default"] = "Default",
					["parrot"] = "Parrot",
					["msbt"] = "MSBT",
					["sct"] = "SCT",
				},
			},
			header1 = {
				order = 200,
				type = 'header',
				hidden = function()
					return self.db.profile.frame ~= "default"
				end,
			},
			unlock = {
				order = 201,
				type = 'boolean',
				name = 'Unlock frame',
				desc = 'Unlocks the message frame',
				get = function()
					return not self.locked
				end,
				set = function()
					self.locked = not self.locked
					self:ToggleMessageFrame()
				end,
				hidden = function()
					return self.db.profile.frame ~= "default"
				end,
			},
			fonttype = {
				order = 202,
				type = 'choice',
				name = 'Font type',
				desc = 'Changes the message font type',
				get = function()
					return self.db.profile.fonttype
				end,
				set = function(fonttype)
					self.db.profile.fonttype = fonttype
					local type, size, outline = self.MessageFrame:GetFont()
					self.MessageFrame:SetFont(SM:Fetch('font', fonttype), math.floor(size+0.5), outline)
					self:Announce("New font type selected")
				end,
				hidden = function()
					return self.db.profile.frame ~= "default"
				end,
				choices = SM:List('font'),
			},
			fontsize = {
				order = 203,
				type = 'choice',
				name = 'Font size',
				desc = 'Changes the message font size',
				get = function()
					return self.db.profile.fontsize
				end,
				set = function(fontsize)
					self.db.profile.fontsize = fontsize
					local type, size, outline = self.MessageFrame:GetFont()
					self.MessageFrame:SetFont(type, fontsize, outline)
					self:Announce("New font size selected")
				end,
				hidden = function()
					return self.db.profile.frame ~= "default"
				end,
				choices = {
					["16"] = "Tiny",
					["24"] = "Small",
					["32"] = "Medium",
					["48"] = "Large",
				},
			},
			fontoutline = {
				order = 204,
				type = 'choice',
				name = 'Font outline',
				desc = 'Changes the message font outline',
				get = function()
					return self.db.profile.fontoutline
				end,
				set = function(fontoutline)
					self.db.profile.fontoutline = fontoutline
					local type, size, outline = self.MessageFrame:GetFont()
					self.MessageFrame:SetFont(type, math.floor(size+0.5), fontoutline)
					self:Announce("New font outline selected")
				end,
				hidden = function()
					return self.db.profile.frame ~= "default"
				end,
				choices = {
					[""] = "None",
					["OUTLINE"] = "Thin",
					["THICKOUTLINE"] = "Thick",
				},
			},
			-- TODO: add messagecolors and messagesounds
		},
	})
	self:CreateMessageFrame()
end

function CombatAlert:OnEnable()
	local PARSER_FILTER = {
		["COMBAT_MSG_CAST"] = {
			eventType = "Cast",
			sourceName_not = false,
			recipientName_not = false,
			abilityName_not = false,
			isBegin = true,
		},
		["COMBAT_MSG_BUFF"] = {
			eventType = "Aura",
			recipientName_not = false,
			abilityName_not = false,
			isBuff = true,
		},
		["COMBAT_MSG_DEBUFF"] = {
			eventType = "Aura",
			recipientName_not = false,
			abilityName_not = false,
			isBuff = false,
		},
		["COMBAT_MSG_DISPEL"] = {
			eventType = "Dispel",
			recipientName_not = false,
			recipientAbilityName_not = false,
		},
		["COMBAT_MSG_FADE"] = {
			eventType = "Fade",
			recipientName_not = false,
			abilityName_not = false,
		},
	}
	self:OnCooldownUpdate()
	self:AddEventListener("PLAYER_REGEN_ENABLED")
	self:AddEventListener("PLAYER_REGEN_DISABLED")
	self:AddEventListener("BAG_UPDATE_COOLDOWN")
	self:AddEventListener("SPELL_UPDATE_COOLDOWN")
	self:AddParserListener(PARSER_FILTER["COMBAT_MSG_CAST"], "OnCastEvent")
	self:AddParserListener(PARSER_FILTER["COMBAT_MSG_BUFF"], "OnBuffEvent")
	self:AddParserListener(PARSER_FILTER["COMBAT_MSG_DEBUFF"], "OnDebuffEvent")
	self:AddParserListener(PARSER_FILTER["COMBAT_MSG_DISPEL"], "OnDispelEvent")
	self:AddParserListener(PARSER_FILTER["COMBAT_MSG_FADE"], "OnFadeEvent")
end

function CombatAlert:OnDisable()
	self:RemoveAllParserListeners()
	self:RemoveAllEventListeners()
	self:RemoveAllCommListeners()
	self:RemoveAllTimers()
end

function CombatAlert:OnCommand(command, text)
	if text and text:find("list%s(.+)") then
		local _, _, pattern = text:find("list%s(.+)")
		self:List(pattern)
	elseif text and text:find("insert%s(.+)") then
		local _, _, pattern = text:find("insert%s(.+)")
		self:Insert(pattern)
	elseif text and text:find("delete%s(.+)") then
		local _, _, pattern = text:find("delete%s(.+)")
		self:Delete(pattern)
	elseif text and text:find("search%s(.+)") then
		local _, _, pattern = text:find("search%s(.+)")
		if self:Search(pattern) then
			self:Print("found")
		else
			self:Print("not found")
		end
	else
		self:Print("Invalid command, view the readme.txt file in this addons directory for valid commands")
	end
end

function CombatAlert:PLAYER_REGEN_ENABLED(caller, event)
	local eventType, combatState = "state", "nocombat"
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s", eventType, combatState)) then
			self:Insert(string.format("known %s %s", eventType, combatState))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s", eventType, combatState))
			end
		end
	end
	if self:Search(string.format("crit %s %s", eventType, combatState)) then
		if self.db.profile.compact then
			self:Announce(string.format("[-combat]"), self.db.profile.messagecolor.state, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("You have exited combat"), self.db.profile.messagecolor.state, MESSAGE_CRITICAL)
		end
	elseif self:Search(string.format("warn %s %s", eventType, combatState)) then
		if self.db.profile.compact then
			self:Announce(string.format("[-combat]"), self.db.profile.messagecolor.state, MESSAGE_WARNING)
		else
			self:Announce(string.format("You have exited combat"), self.db.profile.messagecolor.state, MESSAGE_WARNING)
		end
	elseif self.db.profile.preset or self:Search(string.format("show %s %s", eventType, combatState)) then
		if self.db.profile.compact then
			self:Announce(string.format("[-combat]"), self.db.profile.messagecolor.state, MESSAGE_NORMAL)
		else
			self:Announce(string.format("You have exited combat"), self.db.profile.messagecolor.state, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:PLAYER_REGEN_DISABLED(caller, event)
	local eventType, combatState = "state", "incombat"
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s", eventType, combatState)) then
			self:Insert(string.format("known %s %s", eventType, combatState))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s", eventType, combatState))
			end
		end
	end
	if self:Search(string.format("crit %s %s", eventType, combatState)) then
		if self.db.profile.compact then
			self:Announce(string.format("[+combat]"), self.db.profile.messagecolor.state, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("You have entered combat"), self.db.profile.messagecolor.state, MESSAGE_CRITICAL)
		end
	elseif self:Search(string.format("warn %s %s", eventType, combatState)) then
		if self.db.profile.compact then
			self:Announce(string.format("[+combat]"), self.db.profile.messagecolor.state, MESSAGE_WARNING)
		else
			self:Announce(string.format("You have entered combat"), self.db.profile.messagecolor.state, MESSAGE_WARNING)
		end
	elseif self.db.profile.preset or self:Search(string.format("show %s %s", eventType, combatState)) then
		if self.db.profile.compact then
			self:Announce(string.format("[+combat]"), self.db.profile.messagecolor.state, MESSAGE_NORMAL)
		else
			self:Announce(string.format("You have entered combat"), self.db.profile.messagecolor.state, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:BAG_UPDATE_COOLDOWN(caller, event)
	local slotIDs = {
		"HeadSlot",
		"NeckSlot",
		"ShoulderSlot",
		"BackSlot",
		"ChestSlot",
		"WristSlot",
		"HandsSlot",
		"WaistSlot",
		"LegsSlot",
		"FeetSlot",
		"Finger0Slot",
		"Finger1Slot",
		"Trinket0Slot",
		"Trinket1Slot",
		"MainHandSlot",
		"SecondaryHandSlot",
	}
	for _, slot in ipairs(slotIDs) do
		local slotID = GetInventorySlotInfo(slot)
		if GetInventoryItemLink('player', slotID) then
			local _, _, itemName = string.find(GetInventoryItemLink('player', slotID), "%[(.+)%]")
			local start, duration, hasCooldown = GetInventoryItemCooldown('player', slotID)
			if hasCooldown == 1 and start > 0 and duration > COOLDOWN_MINIMUM and (COOLDOWN_MAXIMUM == nil or duration < COOLDOWN_MAXIMUM) then
				if not self:Search(string.format("cooldowns %s", itemName)) then
					self:Insert(string.format("cooldowns %d %d %d %s %s", start, duration, slotID, 'inventory', itemName))
					local timer = self:HasTimer("COOLDOWN") or 0
					local timediff = duration - (GetTime() - start)
					if timer < timediff then
						self:AddTimer("COOLDOWN", timediff, "OnCooldownUpdate")
					end
				end
			end
		end
	end
	for containerID = 0, 4 do
		for slotID = 1, GetContainerNumSlots(containerID) do
			if GetContainerItemLink(containerID, slotID) then
				local _, _, itemName = string.find(GetContainerItemLink(containerID, slotID), "%[(.+)%]")
				local start, duration, hasCooldown = GetContainerItemCooldown(containerID, slotID)
				if hasCooldown == 1 and start > 0 and duration > COOLDOWN_MINIMUM and (COOLDOWN_MAXIMUM == nil or duration < COOLDOWN_MAXIMUM) then
					if not self:Search(string.format("cooldowns %s", itemName)) then
						self:Insert(string.format("cooldowns %d %d %d %s%d %s", start, duration, slotID, 'container', containerID, itemName))
						local timer = self:HasTimer("COOLDOWN") or 0
						local timediff = duration - (GetTime() - start)
						if timer < timediff then
							self:AddTimer("COOLDOWN", timediff, "OnCooldownUpdate")
						end
					end
				end
			end
		end
	end
end

function CombatAlert:SPELL_UPDATE_COOLDOWN(caller, event)
	local bookTypes = {
		['spell'] = BOOKTYPE_SPELL,
		['pet'] = BOOKTYPE_PET,
	}
	for bookType, bookID in pairs(bookTypes) do
		local spellID = 1
		local spellName = GetSpellName(spellID, bookID)
		while spellName do
			local start, duration, hasCooldown = GetSpellCooldown(spellID, bookID)
			if hasCooldown == 1 and start > 0 and duration > COOLDOWN_MINIMUM and (COOLDOWN_MAXIMUM == nil or duration < COOLDOWN_MAXIMUM) then
				if not self:Search(string.format("cooldowns %s", spellName)) then
					self:Insert(string.format("cooldowns %d %d %d %s %s", start, duration, spellID, bookType, spellName))
					local timer = self:HasTimer("COOLDOWN") or 0
					local timediff = duration - (GetTime() - start)
					if timer < timediff then
						self:AddTimer("COOLDOWN", timediff, "OnCooldownUpdate")
					end
				end
			end
			spellID = spellID + 1
			spellName = GetSpellName(spellID, bookID)
		end
	end
end

function CombatAlert:OnCooldownUpdate()
	for _, cooldown in ipairs(self.db.char.cooldowns) do
		local _, _, _, _, cooldownID, cooldownSource, cooldownName = cooldown:find("(%d+)%s(%d+)%s(%d+)%s(%w+)%s(.+)")
		if cooldownSource == 'pet' then
			local start, duration = GetSpellCooldown(cooldownID, BOOKTYPE_PET)
			if start == 0 or duration == 0 then
				self:Delete(string.format("cooldowns %s", cooldownName))
				self:OnCooldownEvent(cooldownName)
			else
				local timer = self:HasTimer("COOLDOWN") or 0
				local timediff = duration - (GetTime() - start)
				if timer < timediff then
					self:AddTimer("COOLDOWN", timediff, "OnCooldownUpdate")
				end
			end
		elseif cooldownSource == "spell" then
			local start, duration = GetSpellCooldown(cooldownID, BOOKTYPE_SPELL)
			if start == 0 or duration == 0 then
				self:Delete(string.format("cooldowns %s", cooldownName))
				self:OnCooldownEvent(cooldownName)
			else
				local timer = self:HasTimer("COOLDOWN") or 0
				local timediff = duration - (GetTime() - start)
				if timer < timediff then
					self:AddTimer("COOLDOWN", timediff, "OnCooldownUpdate")
				end
			end
		elseif cooldownSource == "inventory" then
			if GetInventoryItemLink('player', slotID) then
				local start, duration = GetInventoryItemCooldown('player', cooldownID)
				if start == 0 or duration == 0 then
					self:Delete(string.format("cooldowns %s", cooldownName))
					self:OnCooldownEvent(cooldownName)
				else
					local timer = self:HasTimer("COOLDOWN") or 0
					local timediff = duration - (GetTime() - start)
					if timer < timediff then
						self:AddTimer("COOLDOWN", timediff, "OnCooldownUpdate")
					end
				end
			end
		elseif cooldownSource:find("container(%d+)") then
			local _, _, containerID = cooldownSource:find("container(%d+)")
			if GetContainerItemLink(containerID, cooldownID) then
				local start, duration = GetContainerItemCooldown(containerID, cooldownID)
				if start == 0 or duration == 0 then
					self:Delete(string.format("cooldowns %s", cooldownName))
					self:OnCooldownEvent(cooldownName)
				else
					local timer = self:HasTimer("COOLDOWN") or 0
					local timediff = duration - (GetTime() - start)
					if timer < timediff then
						self:AddTimer("COOLDOWN", timediff, "OnCooldownUpdate")
					end
				end
			end
 		end
	end
end

function CombatAlert:OnCooldownEvent(cooldownName)
	local eventType = "cooldown"
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s", eventType, cooldownName)) then
			self:Insert(string.format("known %s %s", eventType, cooldownName))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s", eventType, cooldownName))
			end
		end
	end
	if self:Search(string.format("crit %s %s", eventType, cooldownName)) then
		if self.db.profile.compact then
			self:Announce(string.format("[%s]", cooldownName), self.db.profile.messagecolor.cooldown, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("%s ready.", cooldownName), self.db.profile.messagecolor.cooldown, MESSAGE_CRITICAL)
		end
	elseif self:Search(string.format("warn %s %s", eventType, cooldownName)) then
		if self.db.profile.compact then
			self:Announce(string.format("[%s]", cooldownName), self.db.profile.messagecolor.cooldown, MESSAGE_WARNING)
		else
			self:Announce(string.format("%s ready.", cooldownName), self.db.profile.messagecolor.cooldown, MESSAGE_WARNING)
		end
	elseif self.db.profile.preset or self:Search(string.format("show %s %s", eventType, cooldownName)) then
		if self.db.profile.compact then
			self:Announce(string.format("[%s]", cooldownName), self.db.profile.messagecolor.cooldown, MESSAGE_NORMAL)
		else
			self:Announce(string.format("%s ready.", cooldownName), self.db.profile.messagecolor.cooldown, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:OnCastEvent(info)
	local eventType, sourceID, recipientID = "cast"
	if info.sourceID then
		sourceID = info.sourceID:gsub("%d", "")
	else
		sourceID = self:GetUnitInfo(info.sourceName)
	end
	if info.recipientID then
		recipientID = info.recipientID:gsub("%d", "")
	else
		recipientID = self:GetUnitInfo(info.recipientName)
	end
	if not self.db.profile.unknown and (sourceID == "unknown" or recipientID == "unknown") then
		return
	end
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s %s %s", eventType, sourceID, recipientID, info.abilityName)) then
			self:Insert(string.format("known %s %s %s %s", eventType, sourceID, recipientID, info.abilityName))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s %s %s", eventType, sourceID, recipientID, info.abilityName))
			end
		end
	end
	if self:Search(string.format("crit %s %s %s %s", eventType, sourceID, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			self:Announce(string.format("%s casts %s at you", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_CRITICAL)
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s casts %s at your pet", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_CRITICAL)
		elseif sourceID == 'focus' then
			self:Announce(string.format("--- %s casts %s at %s ---", info.sourceName, info.abilityName, info.recipientName or GetUnitName("focustarget")), self.db.profile.messagecolor.cast, MESSAGE_CRITICAL)
		elseif sourceID == 'target' then
			self:Announce(string.format("*** %s casts %s at %s ***", info.sourceName, info.abilityName, info.recipientName or GetUnitName("targettarget")), self.db.profile.messagecolor.cast, MESSAGE_CRITICAL)
		elseif recipientName then
			self:Announce(string.format("%s casts %s at %s", info.sourceName, info.abilityName, info.recipientName), self.db.profile.messagecolor.cast, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("%s casts %s", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_CRITICAL)
		end
	elseif self:Search(string.format("warn %s %s %s %s", eventType, sourceID, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			self:Announce(string.format("%s casts %s at you", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_WARNING)
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s casts %s at your pet", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_WARNING)
		elseif sourceID == 'focus' then
			self:Announce(string.format("--- %s casts %s at %s ---", info.sourceName, info.abilityName, info.recipientName or GetUnitName("focustarget")), self.db.profile.messagecolor.cast, MESSAGE_WARNING)
		elseif sourceID == 'target' then
			self:Announce(string.format("*** %s casts %s at %s ***", info.sourceName, info.abilityName, info.recipientName or GetUnitName("targettarget")), self.db.profile.messagecolor.cast, MESSAGE_WARNING)
		elseif recipientName then
			self:Announce(string.format("%s casts %s at %s", info.sourceName, info.abilityName, info.recipientName), self.db.profile.messagecolor.cast, MESSAGE_WARNING)
		else
			self:Announce(string.format("%s casts %s", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_WARNING)
		end
	elseif (self.db.profile.preset and recipientID == 'player') or self:Search(string.format("show %s %s %s %s", eventType, sourceID, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			self:Announce(string.format("%s casts %s at you", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_NORMAL)
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s casts %s at your pet", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_NORMAL)
		elseif sourceID == 'focus' then
			self:Announce(string.format("--- %s casts %s at %s ---", info.sourceName, info.abilityName, info.recipientName or GetUnitName("focustarget")), self.db.profile.messagecolor.cast, MESSAGE_NORMAL)
		elseif sourceID == 'target' then
			self:Announce(string.format("*** %s casts %s at %s ***", info.sourceName, info.abilityName, info.recipientName or GetUnitName("targettarget")), self.db.profile.messagecolor.cast, MESSAGE_NORMAL)
		elseif recipientName then
			self:Announce(string.format("%s casts %s at %s", info.sourceName, info.abilityName, info.recipientName), self.db.profile.messagecolor.cast, MESSAGE_NORMAL)
		else
			self:Announce(string.format("%s casts %s", info.sourceName, info.abilityName), self.db.profile.messagecolor.cast, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:OnBuffEvent(info)
	local eventType, recipientID = "buff"
	if info.recipientID then
		recipientID = info.recipientID:gsub("%d", "")
	else
		recipientID = self:GetUnitInfo(info.recipientName)
	end
	if not self.db.profile.unknown and recipientID == "unknown" then
		return
	end
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s %s", eventType, recipientID, info.abilityName)) then
			self:Insert(string.format("known %s %s %s", eventType, recipientID, info.abilityName))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s %s", eventType, recipientID, info.abilityName))
			end
		end
	end
	if self:Search(string.format("crit %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[+%s]", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_CRITICAL)
			else
				self:Announce(string.format("You gain %s", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_CRITICAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("Your pet gains %s", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_CRITICAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s gains %s ---", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_CRITICAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s gains %s ***", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("%s gains %s", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_CRITICAL)
		end
	elseif (self.db.profile.preset and (recipientID == 'target' or recipientID == 'focus')) or self:Search(string.format("warn %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[+%s]", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_WARNING)
			else
				self:Announce(string.format("You gain %s", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_WARNING)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("Your pet gains %s", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_WARNING)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s gains %s ---", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_WARNING)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s gains %s ***", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_WARNING)
		else
			self:Announce(string.format("%s gains %s", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_WARNING)
		end
	elseif (self.db.profile.preset and recipientID == 'player') or self:Search(string.format("show %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[+%s]", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_NORMAL)
			else
				self:Announce(string.format("You gain %s", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_NORMAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("Your pet gains %s", info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_NORMAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s gains %s ---", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_NORMAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s gains %s ***", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_NORMAL)
		else
			self:Announce(string.format("%s gains %s", info.recipientName, info.abilityName), self.db.profile.messagecolor.buff, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:OnDebuffEvent(info)
	local eventType, recipientID = "debuff"
	if info.recipientID then
		recipientID = info.recipientID:gsub("%d", "")
	else
		recipientID = self:GetUnitInfo(info.recipientName)
	end
	if not self.db.profile.unknown and recipientID == "unknown" then
		return
	end
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s %s", eventType, recipientID, info.abilityName)) then
			self:Insert(string.format("known %s %s %s", eventType, recipientID, info.abilityName))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s %s", eventType, recipientID, info.abilityName))
			end
		end
	end
	if self:Search(string.format("crit %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[+%s]", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_CRITICAL)
			else
				self:Announce(string.format("You gain %s", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_CRITICAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("Your pet gains %s", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_CRITICAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s gains %s ---", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_CRITICAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s gains %s ***", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("%s gains %s", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_CRITICAL)
		end
	elseif self:Search(string.format("warn %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[+%s]", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_WARNING)
			else
				self:Announce(string.format("You gain %s", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_WARNING)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("Your pet gains %s", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_WARNING)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s gains %s ---", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_WARNING)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s gains %s ***", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_WARNING)
		else
			self:Announce(string.format("%s gains %s", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_WARNING)
		end
	elseif (self.db.profile.preset and recipientID == 'player') or self:Search(string.format("show %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[+%s]", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_NORMAL)
			else
				self:Announce(string.format("You gain %s", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_NORMAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("Your pet gains %s", info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_NORMAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s gains %s ---", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_NORMAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s gains %s ***", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_NORMAL)
		else
			self:Announce(string.format("%s gains %s", info.recipientName, info.abilityName), self.db.profile.messagecolor.debuff, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:OnDispelEvent(info)
	local eventType, sourceID, recipientID = "dispel"
	if info.sourceID then
		sourceID = info.sourceID:gsub("%d", "")
	else
		sourceID = self:GetUnitInfo(info.sourceName)
	end
	if info.recipientID then
		recipientID = info.recipientID:gsub("%d", "")
	else
		recipientID = self:GetUnitInfo(info.recipientName)
	end
	if not self.db.profile.unknown and (sourceID == "unknown" or recipientID == "unknown") then
		return
	end
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s %s %s", eventType, sourceID, recipientID, info.recipientAbilityName)) then
			self:Insert(string.format("known %s %s %s %s", eventType, sourceID, recipientID, info.recipientAbilityName))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s %s %s", eventType, sourceID, recipientID, info.recipientAbilityName))
			end
		end
	end
	if self:Search(string.format("crit %s %s %s %s", eventType, sourceID, recipientID, info.recipientAbilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[-%s]", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_CRITICAL)
			else
				self:Announce(string.format("%s is dispelled from you", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_CRITICAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s is dispelled from your pet", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_CRITICAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s is dispelled from %s ---", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_CRITICAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s is dispelled from %s ***", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("%s is dispelled from %s", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_CRITICAL)
		end
	elseif (self.db.profile.preset and (recipientID == 'target' or recipientID == 'focus')) or self:Search(string.format("warn %s %s %s %s", eventType, sourceID, recipientID, info.recipientAbilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[-%s]", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_WARNING)
			else
				self:Announce(string.format("%s is dispelled from you", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_WARNING)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s is dispelled from your pet", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_WARNING)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s is dispelled from %s ---", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_WARNING)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s is dispelled from %s ***", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_WARNING)
		else
			self:Announce(string.format("%s is dispelled from %s", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_WARNING)
		end
	elseif (self.db.profile.preset and recipientID == 'player') or self:Search(string.format("show %s %s %s %s", eventType, sourceID, recipientID, info.recipientAbilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[-%s]", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_NORMAL)
			else
				self:Announce(string.format("%s is dispelled from you", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_NORMAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s is dispelled from your pet", info.recipientAbilityName), self.db.profile.messagecolor.dispel, MESSAGE_NORMAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s is dispelled from %s ---", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_NORMAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s is dispelled from %s ***", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_NORMAL)
		else
			self:Announce(string.format("%s is dispelled from %s", info.recipientAbilityName, info.recipientName), self.db.profile.messagecolor.dispel, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:OnFadeEvent(info)
	local eventType, recipientID = "fade"
	if info.recipientID then
		recipientID = info.recipientID:gsub("%d", "")
	else
		recipientID = self:GetUnitInfo(info.recipientName)
	end
	if not self.db.profile.unknown and recipientID == "unknown" then
		return
	end
	if self.db.profile.collect then
		if not self:Search(string.format("known %s %s %s", eventType, recipientID, info.abilityName)) then
			self:Insert(string.format("known %s %s %s", eventType, recipientID, info.abilityName))
			if not self.db.profile.silent then
				self:Print(string.format("New %s found: %s %s", eventType, recipientID, info.abilityName))
			end
		end
	end
	if self:Search(string.format("crit %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[-%s]", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_CRITICAL)
			else
				self:Announce(string.format("%s fades from you", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_CRITICAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s fades from your pet", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_CRITICAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s fades from %s ---", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_CRITICAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s fades from %s ***", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_CRITICAL)
		else
			self:Announce(string.format("%s fades from %s", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_CRITICAL)
		end
	elseif self:Search(string.format("warn %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[-%s]", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_WARNING)
			else
				self:Announce(string.format("%s fades from you", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_WARNING)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s fades from your pet", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_WARNING)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s fades from %s ---", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_WARNING)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s fades from %s ***", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_WARNING)
		else
			self:Announce(string.format("%s fades from %s", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_WARNING)
		end
	elseif (self.db.profile.preset and (recipientID == 'player' or recipientID == 'target' or recipientID == 'focus')) or self:Search(string.format("show %s %s %s", eventType, recipientID, info.abilityName)) then
		if recipientID == 'player' then
			if self.db.profile.compact then
				self:Announce(string.format("[-%s]", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_NORMAL)
			else
				self:Announce(string.format("%s fades from you", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_NORMAL)
			end
		elseif recipientID == 'pet' then
			self:Announce(string.format("%s fades from your pet", info.abilityName), self.db.profile.messagecolor.fade, MESSAGE_NORMAL)
		elseif recipientID == 'focus' then
			self:Announce(string.format("--- %s fades from %s ---", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_NORMAL)
		elseif recipientID == 'target' then
			self:Announce(string.format("*** %s fades from %s ***", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_NORMAL)
		else
			self:Announce(string.format("%s fades from %s", info.abilityName, info.recipientName), self.db.profile.messagecolor.fade, MESSAGE_NORMAL)
		end
	end
end

function CombatAlert:Announce(msg, color, type)
	if IsAddOnLoaded("sct") and self.db.profile.frame == "sct" then
		SCT_MSG_FRAME:AddMessage(msg, color.r, color.g, color.b)
	elseif IsAddOnLoaded("Parrot") and self.db.profile.frame == "parrot" then
		Parrot:ShowMessage(msg, "Notification", false, color.r, color.g, color.b)
	elseif IsAddOnLoaded("MikScrollingBattleText") and self.db.profile.frame == "msbt" then
		MikSBT.DisplayMessage(msg, MikSBT.DISPLAYTYPE_NOTIFICATION, false, color.r, color.g, color.b)
	elseif UIOptionsFrameCheckButtons["SHOW_COMBAT_TEXT_TEXT"].value == "1" and self.db.profile.frame == "blizzard" then
		CombatText_AddMessage(msg, COMBAT_TEXT_SCROLL_FUNCTION, color.r, color.g, color.b)
	else
		self.MessageFrame:AddMessage(msg, color.r, color.g, color.b)
	end
	if type == MESSAGE_CRITICAL and UnitFactionGroup('player') == "Horde" then
		PlaySoundFile("Sound\\Doodad\\BellTollHorde.wav")
	elseif type == MESSAGE_CRITICAL and UnitFactionGroup('player') == "Alliance" then
		PlaySoundFile("Sound\\Doodad\\BellTollAlliance.wav")
	elseif type == MESSAGE_WARNING then
		PlaySoundFile("Sound\\Doodad\\BellTollNightElf.wav")
	end
end

function CombatAlert:GetUnitInfo(unitNameRealm)
	local selfunits = { 'player', 'pet', 'focus', 'target', 'mouseover' }
	for _, unit in ipairs(selfunits) do
		local namerealm = GetUnitName(unit, true)
		if namerealm and namerealm == unitNameRealm then
			return unit
		end
	end
	--[[ TODO: add support for more unitids - via UNIT_COMBAT?
	for unit, namerealm in pairs(self.unitnamerealm) do
		if namerealm == unitNameRealm then
			if not UnitPlayerControlled(unit) then
				return 'npc'
			elseif UnitIsFriend(unit, 'player') then
				return 'friend'
			elseif UnitIsEnemy(unit, 'player') then
				return 'enemy'
			end
		end
	end
	--]]
	return "unknown"
end

function CombatAlert:Insert(pattern)
	local _, _, type = pattern:find("(%a+)")
	local _, _, value = pattern:find("%a+%s(.+)")
	if type and self.db.char[type] and value then
		table.insert(self.db.char[type], value)
		table.sort(self.db.char[type])
	end
end

function CombatAlert:Search(pattern)
	local _, _, type = pattern:find("(%a+)")
	local _, _, value = pattern:find("%a+%s(.+)")
	if type and self.db.char[type] and value then
		value = value:gsub("*", ".+")
		for i, v in ipairs(self.db.char[type]) do
			if v:find(value) then
				return true
			end
		end
	end
	return false
end

function CombatAlert:Delete(pattern)
	local _, _, type = pattern:find("(%a+)")
	local _, _, value = pattern:find("%a+%s(.+)")
	if type and self.db.char[type] then
		if not value then
			self.db.char[type] = {}
		else
			value = value:gsub("*", ".+")
			value = value:gsub("\(.+\)", "")
			for i, v in ipairs(self.db.char[type]) do
				if v:find(value) then
					table.remove(self.db.char[type], i)
				end
			end
		end
	end
end

function CombatAlert:List(pattern)
	local _, _, type = pattern:find("(%a+)")
	local _, _, value = pattern:find("%a+%s(.+)")
	if type and self.db.char[type] then
		if value then
			value = value:gsub("*", ".+")
		end
		for i, v in ipairs(self.db.char[type]) do
			if not value or v:find(value) then
				self:Print(string.format("%s %s", type, v))
			end
		end
	end
end

function CombatAlert:ToggleMessageFrame()
	if not self.locked then
		self.MessageFrame:SetMovable(true)
		self.MessageFrame:EnableMouse(true)
		self.MessageFrame:RegisterForDrag("LeftButton")
		self.MessageFrame:SetBackdropColor(1.0, 1.0, 1.0, 0.2)
		self.MessageFrame:SetBackdropBorderColor(1.0, 1.0, 1.0, 0.2)
		self.MessageFrame:SetScript("OnDragStart", function ()
			self.MessageFrame:StartMoving()
		end)
		self.MessageFrame:SetScript("OnDragStop", function()
			self.MessageFrame:StopMovingOrSizing()
			_, _, _, self.db.profile.x, self.db.profile.y = self.MessageFrame:GetPoint()
		end)
	else
		self.MessageFrame:SetMovable(false)
		self.MessageFrame:EnableMouse(false)
		self.MessageFrame:SetBackdropColor(1.0, 1.0, 1.0, 0.0)
		self.MessageFrame:SetBackdropBorderColor(1.0, 1.0, 1.0, 0.0)
	end
end

function CombatAlert:CreateMessageFrame()
	if not self.MessageFrame then
		self.MessageFrame = CreateFrame("MessageFrame")
		self.MessageFrame:SetInsertMode("BOTTOM")
		self.MessageFrame:SetFrameStrata("HIGH")
		self.MessageFrame:SetJustifyH("CENTER")
		self.MessageFrame:SetToplevel(true)
		self.MessageFrame:SetHeight(100)
		self.MessageFrame:SetWidth(600)
		self.MessageFrame:SetFading(true)
		self.MessageFrame:SetTimeVisible(3)
		self.MessageFrame:SetFadeDuration(3)
		self.MessageFrame:SetPoint("CENTER", UIParent, "CENTER", self.db.profile.x, self.db.profile.y)
		self.MessageFrame:SetFont(SM:Fetch('font', self.db.profile.fonttype), self.db.profile.fontsize, self.db.profile.fontoutline)
		self.MessageFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeSize = 16, edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true, tileSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
		self:ToggleMessageFrame()
	end
end
